PROGRAM avalanche

!****************
!* USEd Modules *
!****************
USE types
USE linalg
USE ieulidec
USE parameters
USE work

!*******************
!* Local variables *
!*******************
IMPLICIT NONE

REAL(kind = dp)          :: y(1)                    !initial value
REAL(KIND = dp)          :: integral1,integral2     !integral with two different
                                                    !methods
REAL(KIND = dp)          :: slope(1),integral       !auxiliary variables
REAL(KIND = dp), POINTER :: solution(:,:),sol(:,:)  !the solution vector
REAL(KIND = dp)          :: nstx=0                  !the root of v(t)
REAL(KIND = dp)          :: est=0,estimator=HUGE(1.0_dp)       !error estimates
REAL(KIND = dp)          :: global_rel=1.0_dp,tol=HUGE(1.0_dp) !tolerances
REAL(KIND = dp)          :: estint, estidec                    !auxiliary variables
INTEGER(KIND = i4b)      :: ifail,j                 !auxiliary variables

!**********************************
!* INTERFACE for USEd subroutines *
!**********************************
INTERFACE

FUNCTION fcn(t, y)
 USE types, ONLY:dp
 IMPLICIT NONE
 REAL (KIND=dp), INTENT(IN) :: t, y(:)
 REAL (KIND=dp) :: fcn(size(y))
END FUNCTION fcn

FUNCTION dfcn(t, y)
 USE types, ONLY :dp
 IMPLICIT NONE
 REAL (KIND=dp), INTENT(IN):: t, y(:)
 REAL (KIND=dp) :: dfcn(size(y), size(y))
END FUNCTION dfcn

FUNCTION sint(a,b,y,estint,estidec)
 USE TYPES
 USE IEULIDEC
 IMPLICIT NONE
 REAL(KIND = dp), INTENT(IN) :: a, b
 REAL(KIND = dp), INTENT(INOUT):: y(:)
 REAL(KIND = dp), INTENT(OUT):: estint, estidec
 REAL(KIND = dp) :: sint
END FUNCTION sint
END INTERFACE

!*****************
!* program code  *
!*****************

!Initializations
!---------------
  CALL init
  y(1)=ys

!Display information about model parameters
!-------------------------------
  WRITE (*,*) "model parameters:"
  WRITE (*,*) "-----------------"
  WRITE (*,FMT = '(A,f20.14)') "rho_bar                             =",rho_bar
  WRITE (*,FMT = '(A,f20.14)') "g                                   =",g
  WRITE (*,FMT = '(A,f20.14)') "h_bar                               =",h_bar
  WRITE (*,FMT = '(A,f20.14)') "psi                                 =",psi
  WRITE (*,FMT = '(A,f20.14)') "h_null                              =",h_null
  WRITE (*,FMT = '(A,f20.14)') "v_null                              =",v_null
  WRITE (*,FMT = '(A,f20.14)') "psi_null                            =",psi_null
  WRITE (*,FMT = '(A,f20.14)') "mu                                  =",mu
  WRITE (*,FMT = '(A,f20.14)') "rho_t                               =",rho_t
  WRITE (*,FMT = '(A,f20.14)') "C_D                                 =",C_D
  WRITE (*,FMT = '(A,f20.14)') "k_p                                 =",k_p
  WRITE (*,FMT = '(A,f20.14)') "phi                                 =",phi
  WRITE (*,*)
  WRITE (*,FMT = '(A,f20.14)') "G_null                              =",G_null
  WRITE (*,FMT = '(A,f20.14)') "V                                   =",V
  WRITE (*,FMT = '(A,f20.14)') "D_null                              =",D_null

  WRITE (*,*)
  WRITE (*,*) "General settings:"
  WRITE (*,*) "-----------------"

  WRITE (*,FMT='(A,G20.10E2)')  "EPS (maschine accuracy)             =",epsilon(h)
  WRITE (*,FMT = '(A,f20.14)')  "initial step size                 h =",h
  WRITE (*,*)

!program start
!-------------
  
  integral=0
  WRITE (*,*) "Calculating, please wait ..." 
  CALL calc(fcn, y, h, dfcn, sol, nstx, integral, est)  ! calls local connection strategy
  WRITE (*,*) "... finished"                            ! from module work
  WRITE (*,*)
  WRITE (*,*) "The solution is written to file ", filename
  OPEN(unit=1,FILE=filename,ACCESS='sequential')
  DO j = 0, SIZE(sol)-1                                 ! positive values of v(t) are  
    WRITE (1,*) j*h/4,"   ",sol(j+1,1)                  ! written to file
    IF (sol(j+2,1)<0) EXIT                              ! note that the last (negative)
  END DO                                                ! value is not included in sol  
  CLOSE(1)

!print the results
!-----------------
  WRITE (*,*)
  WRITE (*,*) "The results of the local connection strategy are"
  WRITE (*,*) "-------------------------------------------------"
  WRITE (*,FMT = '(A,f20.14)') "The root is at                      :",nstx
  WRITE (*,FMT = '(A,f20.14)') "The integral is                     :",integral
  WRITE (*,FMT='(A,G20.10E2)') "The global error estimate is        :",est

!check for second run with global connection strategy
!----------------------------------------------------
  IF (.not. accurate) stop
  y(1)=ys
  WRITE (*,*)
  WRITE (*,*) "You have chosen to rerun the calculation for higher accuracy."
  WRITE (*,*) "Calculating, please wait ..."
  nstx=h*CEILING(nstx/h)

estimator=tol           ! resetting INOUT parameters for ivp_ieuleridec
global_rel=1.0_dp
h0=h/4.0_dp             ! length of one Euler step

! call of IDeC routine from module ieulidec

  call ivp_ieuleridec(fcn, dfcn, 0._dp, y, nstx, .false., h0, estimator, &
           global_rel, 10._dp**(-12), 4, ifail, solution=solution, &
           order_poly=4, use_damping=.true., use_stepcontrol=.false.)

  WRITE (*,*) "  ... finished"
  
!Calculate root and integral for improved solution
!-------------------------------------------------
  j=0
  nstx=0
  integral1=0
  integral2=0
  integral=0
  DO WHILE ((j<size(solution)) .AND. (nstx .EQ. 0))
    IF (solution(j,1)*solution(j+1,1) .le. 0) THEN !has the sign changed?
      nstx = (j*h/4)-solution(j,1)*((h/4)/(solution(j+1,1)-solution(j,1)))
    END IF
    IF ((mod(j,4).eq.0) .AND. (j>0)) THEN
      integral1=integral1+2*h/4*(7*solution(j-4,1)+32*solution(j-3,1)+&
                12*solution(j-2,1)+32*solution(j-1,1)+7*solution(j,1))/45
      integral2=integral2+h/4*(solution(j-4,1)+4*solution(j-3,1)+&
                2*solution(j-2,1)+4*solution(j-1,1)+solution(j,1))/3 
    END IF
    j=j+1
  END DO

! auxiliary integration of the values from the beginning 
! of the last interval to the root of v(t)

  integral=sint(h*FLOOR(1._dp*(j-1)/4),nstx,solution(FLOOR(1._dp*j/4)*4,:),&
                                                           estint,estidec)
  IF (integral>0._dp) THEN
    integral=integral+integral1
  ELSE
    integral=integral1
  END IF
  slope=abs(fcn(nstx,(/0._dp/)))      ! used for error estimation of root
  est=abs(integral1-integral2)+estint+estimator*h*(j-mod(j,4))/4+&       ! error of X_R
      estidec*(nstx+abs((estimator+estidec)/slope(1))-h*(j-mod(j,4))/4)
  WRITE (*,FMT = '(A)') "/---------------------------------------------------------\"
  WRITE (*,FMT = '(A)') "|   The final results of the global connection strategy   |"
  WRITE (*,FMT = '(A)') "|---------------------------------------------------------|"
  WRITE (*,FMT = '(A,f20.14,A)')  "| The root is at                    :", nstx, " |"
  WRITE (*,FMT = '(A,G20.10E2,A)') "| Error estimation for the root     :",&
                                              abs((estimator+estidec)/slope(1)), " |"
  WRITE (*,FMT = '(A,f20.14,A)') "| The integral is                   :", integral,&
                                                                                 " |"
  WRITE (*,FMT = '(A,G20.10E2,A)') "| Error estimate for the integral :",&
                                                abs(integral1-integral2)+estint, " |"
  WRITE (*,FMT = '(A,G20.10E2,A)') "| The global error estimate is      :", est, " |"
  WRITE (*,FMT = '(A)') "\---------------------------------------------------------/"
  WRITE (*,*)
  WRITE (*,*) "The solution is written to file ", filename
  OPEN(unit=1,FILE=filename,ACCESS='sequential')
  DO j = 0, SIZE(solution)-1                 ! positive values of v(t) are written 
    WRITE (1,*) j*h/4,"   ",solution(j,1)    ! to file
    IF (solution(j+1,1)<0) EXIT              ! solution contains all points of the 
  END DO                                     ! solution 
  CLOSE(1)


!clear memory
!-------------------
  DEALLOCATE(solution)
END PROGRAM AVALANCHE
