MODULE parameters

!the model parameters and step-size are set
!------------------------------------------

USE types

REAL (KIND = dp)     :: pi       =   3.1415926535897932385_dp
REAL (KIND = dp)     :: g        =   9.81_dp
!model parameters
!----------------
REAL (KIND = dp)     :: rho_bar  = 200._dp
REAL (KIND = dp)     :: h_bar    =   3_dp
REAL (KIND = dp)     :: psi
REAL (KIND = dp)     :: h_null   =   2._dp
REAL (KIND = dp)     :: v_null   =  35._dp
REAL (KIND = dp)     :: psi_null
REAL (KIND = dp)     :: mu       =   0.155_dp
REAL (KIND = dp)     :: rho_t    =  10._dp
REAL (KIND = dp)     :: C_D      =   0.01_dp
REAL (KIND = dp)     :: phi
REAL (KIND = dp)     :: k_p      =   1._dp
REAL (KIND = dp)     :: G_null
REAL (KIND = dp)     :: V
REAL (KIND = dp)     :: D_null

!initial step-size
!-----------------
REAL (KIND = dp)     :: h0,h        = 2._dp**(-7)

!initial value - will be set automatically
!-----------------------------------------
REAL (KIND = dp)     :: ys

!------------------------------------------------------
!run calculations with global connection strategy
!.true. means yes. This takes as much time as it took
!       for the local connection strategy
!.false. means no, this halves the runtime
!------------------------------------------------------
LOGICAL              :: accurate = .true.

!The filename for solution output
!--------------------------------
CHARACTER(LEN=15)    :: filename = 'avalanche.dat'

CONTAINS

SUBROUTINE init

!calculates the parameters in the differential equation

psi      = -pi/6._dp
psi_null = pi/20._dp
phi      = 5*pi/36._dp

IF (phi .GE. psi_null) THEN
 k_p=(cos(psi_null)+sqrt(cos(psi_null)**2_dp-cos(phi)**2_dp))/&
     (cos(psi_null)-sqrt(cos(psi_null)**2_dp-cos(phi)**2_dp))
ELSE
 k_p=1.0_dp
END IF

G_null = g*(mu*COS(psi)-SIN(psi))
V      = v_null*COS(psi_null-psi)*(1+k_p*g*h_null*COS(psi_null)/(2*v_null**2))
D_null = rho_t*.5_dp*C_D/(rho_bar*h_bar)
ys     = V
END SUBROUTINE init

END MODULE parameters
