	MODULE work

	!calculates the solution using the local connection strategy
	!-----------------------------------------------------------

	!***************
	!* USEd Modules *
	!***************
	USE types
	USE linalg
	USE ieulidec
	USE parameters

	!*************
	!* Variables *
	!*************
	IMPLICIT NONE

	CONTAINS

	SUBROUTINE calc(fcn, y, h, dfcn, solution, nstx, integral, est)
	 USE types
	 IMPLICIT NONE
	   INTERFACE
	      FUNCTION fcn(t, y)     ! right-hand side of the IVP, y'(t) = fcn(t, y)
		USE TYPES, ONLY : dp
		IMPLICIT NONE
		REAL(KIND = dp), INTENT(IN) :: t, y(:)
		REAL(KIND = dp) :: fcn(SIZE(y))
	      END FUNCTION fcn
	      FUNCTION dfcn(t, y)    ! Jacobian of fcn(t, y)
		USE TYPES, ONLY : dp
		IMPLICIT NONE
		REAL(KIND = dp), INTENT(IN) :: t, y(:)
		REAL(KIND = dp) :: dfcn(SIZE(y), SIZE(y))
	      END FUNCTION dfcn
	      FUNCTION sint(a, b, y, estint, estidec)
		USE types
		USE ieulidec
		IMPLICIT NONE
		REAL(KIND = dp), INTENT(IN) :: a, b
		REAL(KIND = dp), INTENT(INOUT):: y(:)
		REAL(KIND = dp), INTENT(OUT):: estint, estidec
		REAL(KIND = dp) :: sint
	      END FUNCTION sint  
	   END INTERFACE

	! Local variables
	!-----------------
	   REAL(KIND = dp), INTENT(INOUT) :: y(:)   !used for inital values
	   REAL(KIND = dp) :: h,h0                  !step size
	   REAL(KIND = dp), POINTER :: solution(:,:)!solution vector
	   REAL(KIND = dp), INTENT(OUT) :: nstx     !the root of v(t)
	   REAL(KIND = dp), INTENT(OUT) :: integral !Integral on [0,nstx]
	   REAL(KIND = dp), INTENT(OUT) :: est      !error estimate
	   LOGICAL :: linear                        !flag for ieulidec
	   INTEGER(KIND = i4b) :: method, pord      !see ieulidec.f90
	   INTEGER(KIND = i4b) :: ifail             !error flag for ieulidec
	   REAL(KIND = dp) :: a, b, Newton_tol      !auxiliary variables
	   REAL(KIND = dp) :: slope(1:1), h_min     !auxiliary variables
	   REAL(KIND = dp) :: integral1, integral2  !integral auxiliary variables
	   REAL(KIND = dp) :: global_rel=1.0_dp,tol=HUGE(1.0_dp) !tolerances, see ieulidec.f90
	   REAL(KIND = dp) :: estimator=HUGE(1.0_dp),estint      !error estimates
	   REAL(KIND = dp) :: estidec               !estimate auxiliary variable
	   INTEGER(KIND = i4b) :: i,w=0             !auxiliary variables

	   TYPE TGlobalsol                          ! auxiliary structure
	     REAL(KIND = dp), pointer :: daten(:,:) ! to build solution vector
	     TYPE(TGlobalsol), POINTER :: next
	   END TYPe TGlobalsol
	   TYPE (TGlobalsol), pointer :: globalsol
	   TYPE (TGlobalsol), pointer :: hp
	  
	!Initialization
	!--------------

	  pord = 4
	  method = 3
	  a = 0._dp
	  b = h
	  nstx = 0
	  ifail = 0
	  est = 0
	  estimator = 0
	  integral = 0
	  integral1 = 0
	  integral2 = 0
	  w = 0
	  NULLIFY(globalsol)
	  NULLIFY(hp)
	  linear = .false.

	!Main loop for local connection strategy
	!---------------------------------------
	  DO WHILE ((nstx==0) .AND. (ifail==0))

	estimator=tol        ! reset INOUT parameters for ivp_ieuleridec
	global_rel=1.0_dp
	h0=h                 ! stepsize is reduced to h/4 in the subroutine

	! IVP integration on one Zadunaisky interval

	  call ivp_ieuleridec(fcn, dfcn, a, y, b, .false., h0, estimator, &
		   global_rel, 10._dp**(-12), 4, ifail, solution=solution, &
		   order_poly=4, use_damping=.true., use_stepcontrol=.false.)

	    IF (estimator>est) est=estimator

	    IF (ifail .ne. 0) write (*,*) "ifail: ", ifail, "delta: ", h-b+a

  !concatenate solution vector
  !---------------------------
    ALLOCATE(hp)
    ALLOCATE(hp%daten(size(solution,1),size(solution,2)))
    hp%daten=solution
    hp%next=>globalsol
    globalsol=>hp 
    w=w+1

  !check for sign change    
  !---------------------
    DO i=0, SIZE(solution(:,1))-2
      IF (solution(i,1)*solution(i+1,1) .lt. 0) THEN !has the sign changed?
        nstx = (a+i*h/pord)-solution(i,1)*(h/pord/(solution(i+1,1)-&
                                             solution(i,1))) !interpolate nstx
      END IF
    END DO

  !perform integration
  !----------------------
    IF (nstx .EQ. 0) THEN
       integral1=integral1+2*h/pord*(7*solution(0,1)+32*solution(1,1)+&
                12*solution(2,1)+32*solution(3,1)+7*solution(4,1))/45
       integral2=integral2+h/pord*(solution(0,1)+4*solution(1,1)+&
                 2*solution(2,1)+4*solution(3,1)+solution(4,1))/3 
    END IF

  !jump to the next interval
  !-------------------------
    a=a+h
    b=a+h
    y=solution(pord,1)
    estimator=0
  END DO 

  !integration of the last interval
  !--------------------------------
  integral=sint(a-h, nstx, solution(0,:), estint, estidec)
  DEALLOCATE(solution)
  IF (ASSOCIATED(globalsol)) THEN
    ALLOCATE(solution(w*(SIZE(globalsol%daten,1)-1),SIZE(globalsol%daten,2)))
  ELSE
    WRITE (*,*) "Fatal error!"
  END IF

  !generate solution vector
  !--------------------------
  w=0
  hp=>globalsol
  DO WHILE (ASSOCIATED(hp)) 
    w=w+1
    solution(SIZE(solution)-w*(SIZE(hp%daten,1)-1)+1:SIZE(solution)-&
         (w-1)*(SIZE(hp%daten,1)-1),:)=hp%daten(1:SIZE(hp%daten)-1,:)
    hp=>hp%next       ! note that solution does not contain the endpoint
  END DO              ! because v(t) is negative at this point anyway

  !global error for the method
  !---------------------------
  slope=ABS(fcn(nstx,(/0._dp/)))  ! for error estimate of the root of v(t)
  estimator=ABS(integral1-integral2)+estint+est*ABS(a-h)+&  ! total error in X_R
            estidec*(nstx+ABS((est+estidec)/slope(1))-a+h)  
  est=estimator

  IF (integral>0._dp) THEN
    integral=integral+integral1
  ELSE
    integral=integral1
  END IF

END SUBROUTINE calc

END MODULE work

FUNCTION sint(a,b,y,estint,estidec)

!integration on the last interval
!--------------------------------

  USE types
  USE ieulidec
  IMPLICIT NONE
  REAL(KIND=dp), INTENT(IN) :: a, b
  REAL(KIND=dp), INTENT(INOUT)::y(:)
  REAL(KIND=dp), INTENT(OUT) ::estint, estidec
  REAL(KIND=dp) :: global_rel=1.0_dp,h,h0,tol=HUGE(1.0_dp)
  INTEGER(KIND=i4b) ::ifail
  REAL(KIND=dp) :: sint, help, estimator, help2
  REAL(KIND=dp), POINTER :: solution(:,:)
  
  INTERFACE

  FUNCTION fcn(t, y)        !right-hand side of the differential equation
    USE types, ONLY:dp
    IMPLICIT NONE
    REAL (KIND=dp), INTENT(IN) :: t, y(:)
    REAL (KIND=dp) :: fcn(size(y))
  END FUNCTION fcn
      
  FUNCTION dfcn(t, y)       !Jacobian of fcn 
    USE types, ONLY :dp
    IMPLICIT NONE
    REAL (KIND=dp), INTENT(IN):: t, y(:)
    REAL (KIND=dp) :: dfcn(size(y), size(y))
  END FUNCTION dfcn
  END INTERFACE

  h=b-a


estimator=tol        ! reset INOUT parameters for ivp_ieuleridec 
global_rel=1.0_dp
h0=h                 ! step size is adjusted correctly in the subroutine     

! solution from the start of the last interval to the root of v(t)

  call ivp_ieuleridec(fcn, dfcn, a, y, b, .false., h0, estimator, &
           global_rel, 10._dp**(-12), 4, ifail, solution=solution, &
           order_poly=4, use_damping=.true., use_stepcontrol=.false.)

  help=2._dp*h/4*(7*solution(0,1)+32*solution(1,1)+&
                  12*solution(2,1)+32*solution(3,1)+7*solution(4,1))/45
  help2=h/4*(solution(0,1)+4*solution(1,1)+&
                   2*solution(2,1)+4*solution(3,1)+solution(4,1))/3
  estint=abs(help-help2)
  estidec=estimator
  sint=help
END FUNCTION sint

FUNCTION fcn(t, y)  !right-hand side of the differential equation
 USE types, ONLY:dp
 USE parameters
 IMPLICIT NONE
 REAL (KIND=dp), INTENT(IN) :: t, y(:)
 REAL (KIND=dp) :: fcn(size(y))
 
  fcn=(/ -y(1)/t-D_null*y(1)*y(1)+V/t-G_null/)
END FUNCTION fcn

FUNCTION dfcn(t, y)  !Jacobian of the rigth-hand side
 USE types, ONLY :dp
 USE parameters
 IMPLICIT NONE
 REAL (KIND=dp), INTENT(IN):: t,y(:)
 REAL (KIND=dp) :: dfcn(size(y),size(y))

 dfcn(1,:)=(/ -1/t-2.*D_null*y(1) /)
END FUNCTION dfcn
